==> derby-0.0.10.tgz: package/lib/History.js <==
var History, cancelRender, qs, renderRoute, routePath, win, winHistory, winLocation;
qs = require('qs');
win = window;
winHistory = win.history;
winLocation = win.location;
winHistory.replaceState({
  render: true,
  method: 'get'
}, null, winLocation.href);
History = module.exports = function(_routes, page) {
  this._routes = _routes;
  page.history = this;
  this._page = page;
};
History.prototype = {
  push: function(url, render, e) {
    return this._update('pushState', url, render, e);
  },
  replace: function(url, render, e) {
    return this._update('replaceState', url, render, e);
  },
  back: function() {
    return winHistory.back();
  },
  forward: function() {
    return winHistory.forward();
  },
  go: function(i) {
    return winHistory.go(i);
  },
  _update: function(historyMethod, url, render, e) {
    var body, el, form, method, name, override, query, _i, _len, _ref;
    if (e && e.type === 'submit') {
      form = e.target;
      query = [];
      _ref = form.elements;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        el = _ref[_i];
        if (name = el.name) {
          query.push(encodeURIComponent(name) + '=' + encodeURIComponent(el.value));
          if (name === '_method') {
            override = el.value.toLowerCase();
            if (override === 'delete') {
              override = 'del';
            }
          }
        }
      }
      query = query.join('&');
      if (form.method.toLowerCase() === 'post') {
        method = override || 'post';
        body = qs.parse(query);
      } else {
        method = 'get';
        url += '?' + query;
      }
    } else {
      method = 'get';
    }
    winHistory[historyMethod]({
      render: render,
      method: method
    }, null, url);
    if (render) {
      return renderRoute(url, body, this._page, this._routes[method], 0, form, e);
    }
  },
  _onClickLink: function(e) {
    var i, path, url;
    url = e.target.href;
    if (~(i = url.indexOf('#')) && url.substr(0, i) === winLocation.href.replace(/#.*/, '')) {
      return;
    }
    if (path = routePath(url)) {
      return this.push(path, true, e);
    }
  },
  _onSubmitForm: function(e) {
    var form, path;
    form = e.target;
    if (!(path = routePath(form.action)) || form._forceSubmit || form.enctype === 'multipart/form-data') {
      return;
    }
    return this.push(path, true, e);
  },
  _onPop: function(e) {
    var state;
    if (!((state = e.state) && state.render)) {
      return;
    }
    return renderRoute(winLocation.pathname, null, this._page, this._routes[state.method], 0, e);
  }
};
cancelRender = function(url, form, e) {
  if (e) {
    return;
  }
  if (form) {
    form._forceSubmit = true;
    return form.submit();
  } else {
    return win.location = url;
  }
};
renderRoute = function(url, body, page, routes, i, form, e) {
  var j, match, name, next, params, path, query, route, _len, _ref, _ref2;
  url = url.replace(/#.*/, '');
  console.log('requesting', url);
  _ref = url.split('?'), path = _ref[0], query = _ref[1];
  while (route = routes[i++]) {
    if (!(match = route.match(path))) {
      continue;
    }
    if (e) {
      e.preventDefault();
    }
    params = {
      url: url,
      body: body,
      query: query ? qs.parse(query) : {}
    };
    _ref2 = route.keys;
    for (j = 0, _len = _ref2.length; j < _len; j++) {
      name = _ref2[j].name;
      params[name] = match[j + 1];
    }
    next = function(err) {
      if (err != null) {
        return cancelRender(url, form);
      }
      return renderRoute(url, body, page, routes, i, form);
    };
    try {
      route.callbacks(page, page.model, params, next);
    } catch (err) {
      cancelRender(url, form);
    }
    return;
  }
  return cancelRender(url, form, e);
};
routePath = function(url) {
  var match;
  match = /^(https?:)\/\/([^\/]+)(.*)/.exec(url);
  return match && match[1] === winLocation.protocol && match[2] === winLocation.host && match[3];
};

==> derby-0.1.0.tgz: package/lib/History.js <==
var History, cancelRender, qs, renderRoute, routePath, win, winHistory, winLocation;
qs = require('qs');
win = window;
winHistory = win.history;
winLocation = win.location;
if (winHistory.replaceState) {
  winHistory.replaceState({
    render: true,
    method: 'get'
  }, null, winLocation.href);
}
History = module.exports = function(_routes, page) {
  this._routes = _routes;
  if (!winHistory.pushState) {
    this.push = this.replace = function() {};
  }
  page.history = this;
  this._page = page;
};
History.prototype = {
  push: function(url, render, e) {
    return this._update('pushState', url, render, e);
  },
  replace: function(url, render, e) {
    return this._update('replaceState', url, render, e);
  },
  back: function() {
    return winHistory.back();
  },
  forward: function() {
    return winHistory.forward();
  },
  go: function(i) {
    return winHistory.go(i);
  },
  _update: function(historyMethod, url, render, e) {
    var body, el, form, method, name, override, query, _i, _len, _ref;
    if (e && e.type === 'submit') {
      form = e.target;
      query = [];
      _ref = form.elements;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        el = _ref[_i];
        if (name = el.name) {
          query.push(encodeURIComponent(name) + '=' + encodeURIComponent(el.value));
          if (name === '_method') {
            override = el.value.toLowerCase();
            if (override === 'delete') {
              override = 'del';
            }
          }
        }
      }
      query = query.join('&');
      if (form.method.toLowerCase() === 'post') {
        method = override || 'post';
        body = qs.parse(query);
      } else {
        method = 'get';
        url += '?' + query;
      }
    } else {
      method = 'get';
    }
    winHistory[historyMethod]({
      render: render,
      method: method
    }, null, url);
    if (render) {
      return renderRoute(url, body, this._page, this._routes[method], 0, form, e);
    }
  },
  _onClickLink: function(e) {
    var i, path, url;
    url = e.target.href;
    if (~(i = url.indexOf('#')) && url.substr(0, i) === winLocation.href.replace(/#.*/, '')) {
      return;
    }
    if (path = routePath(url)) {
      return this.push(path, true, e);
    }
  },
  _onSubmitForm: function(e) {
    var form, path;
    form = e.target;
    if (!(path = routePath(form.action)) || form._forceSubmit || form.enctype === 'multipart/form-data') {
      return;
    }
    return this.push(path, true, e);
  },
  _onPop: function(e) {
    var state;
    if (!((state = e.state) && state.render)) {
      return;
    }
    return renderRoute(winLocation.pathname, null, this._page, this._routes[state.method], 0, null, e);
  }
};
cancelRender = function(url, form, e) {
  if (e) {
    return;
  }
  if (form) {
    form._forceSubmit = true;
    return form.submit();
  } else {
    return win.location = url;
  }
};
renderRoute = function(url, body, page, routes, i, form, e) {
  var j, match, name, next, params, path, query, route, _len, _ref, _ref2;
  url = url.replace(/#.*/, '');
  _ref = url.split('?'), path = _ref[0], query = _ref[1];
  while (route = routes[i++]) {
    if (!(match = route.match(path))) {
      continue;
    }
    if (e) {
      e.preventDefault();
    }
    params = {
      url: url,
      body: body,
      query: query ? qs.parse(query) : {}
    };
    _ref2 = route.keys;
    for (j = 0, _len = _ref2.length; j < _len; j++) {
      name = _ref2[j].name;
      params[name] = match[j + 1];
    }
    next = function(err) {
      if (err != null) {
        return cancelRender(url, form);
      }
      return renderRoute(url, body, page, routes, i, form);
    };
    try {
      route.callbacks(page, page.model, params, next);
    } catch (err) {
      cancelRender(url, form);
    }
    return;
  }
  return cancelRender(url, form, e);
};
routePath = function(url) {
  var match;
  match = /^(https?:)\/\/([^\/]+)(.*)/.exec(url);
  return match && match[1] === winLocation.protocol && match[2] === winLocation.host && match[3];
};
